/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.core;

/**
 * @author Wei LU
 * @version 1.0
 */
import java.util.ArrayList;
import java.io.Serializable;

public class CandidateSet implements Serializable {

	private static final long serialVersionUID = 7253013461605041884L;
	
	private int _candidatesId;

    private ArrayList<Candidate> _candidates;

    private CandidateSet _corrCandidates;

    public CandidateSet(int candidatesId) {
        this._candidatesId = candidatesId;
        this._candidates = new ArrayList<Candidate>();
    }

    public void setCorrectCandidates(CandidateSet corrCandidates) {
        this._corrCandidates = corrCandidates;
    }

    public boolean addCandidate(Candidate candidate) {
        return this._candidates.add(candidate);
    }

    public int getCandidtesId() {
        return this._candidatesId;
    }

    public int size() {
        return this._candidates.size();
    }

    public Candidate getCandidateAt(int index) {
        return this._candidates.get(index);
    }

    public ArrayList<Candidate> getCandidates() {
        return this._candidates;
    }

    public CandidateSet getCorrectCandidates() {
        return this._corrCandidates;
    }

}
