/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.core;

/**
 * @author LU, Wei
 * @version 1.0
 */

import java.util.ArrayList;

import hybridtree.data.Example;
import hybridtree.data.GProduction;
import hybridtree.data.Production;
import hybridtree.data.Word;

public abstract class DeriveTable {

    protected int _exampleId;

    protected boolean _debugMode = false;

    /**
     * the NL sentence
     */
    protected String _sentence;

    /**
     * the NL words
     */
    protected ArrayList<Word> _words;

    /**
     * length of the NL sentence
     */
    protected int _N;
    /**
     * the parameter
     */
    protected Param _parameter;

    /**
     * whether this example should be ignored.
     */
    protected boolean _ignoreFlag = false;

    /**
     * the name of the corpus
     */
    protected String _corpus = "";

    /**
     * Constructor
     * @param example the Example
     * @param parameter the parameter
     */
    protected DeriveTable(Example example, Param parameter) {

        this._sentence = example.getNL();
        this._words = example.getWords();
        this._N = this._words.size();
        this._parameter = parameter;
        this._exampleId = example.getExampleId();

    }

    /**
     * Get the word at a specified position
     * @param wPos the position of the word
     * @return Word
     */
    protected Word getWord(int wPos) {
        return this._words.get(wPos);
    }

    protected Word getChildWord(int cIndex) {
        return cIndex == 0 ? Word.CHILD_A : Word.CHILD_B;
    }

    public double computeRuleTypeProb(GProduction prod, int patternType) {
        return this._parameter.getPatternProb(prod, patternType);
    }

    public double computeRuleTypeProb(Production prod, int patternType) {
        return this._parameter.getPatternProb(prod.getGProduction(), patternType);
    }

    public boolean ignored() {
        return this._ignoreFlag;
    }

    /**
     * Set the corpus
     * @param corpus the name of the corpus
     */
    public void setCorpus(String corpus) {
        this._corpus = corpus;
    }

    /**
     * Get the natural language sentence
     * @return the natural language sentence
     */
    public String getSentence() {
        return this._sentence;
    }

    /**
     * Compute the MRL Language Model's Probabilities
     * @param prod Production
     * @param cIndex int
     * @param child Production
     * @return double
     */
    protected double computeMRLLMProb(Production prod, int cIndex, Production child) {
        //warning: luwei changed this line..
        return 1.0;
        //return this.computeMRLLMProb(prod.getGProduction(), cIndex, child.getGProduction());
    }

    /**
     * Compute the MRL Language Model's Probabilities
     * @param prod GProduction
     * @param cIndex int
     * @param child GProduction
     * @return double
     */
    protected double computeMRLLMProb(GProduction prod, int cIndex, GProduction child) {
        //note: changed by me on 3 Dec for testing on atis corpus...
        /*
        if (child.isIdent() || child.isNum() || child.isUnum() || child.isConst()) {
            return 1.0;
        }
        */
       if(child.isConstant()){
           return 1.0;
       }
        return this._parameter.getMRLMProb(prod, cIndex, child);
    }

}


