/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package hybridtree.core;

/**
 * @author Wei LU
 * @version 1.0
 */
import java.util.ArrayList;

import hybridtree.data.Word;

import java.io.Serializable;


public abstract class HybridNode implements Serializable {

	private static final long serialVersionUID = 4072818967866473137L;

	protected HybridNode _parent;

    protected ArrayList<HybridNode> _children;

    public HybridNode() {
        this._children = new ArrayList<HybridNode>();
    }

    public static WordHybridNode createWordHybridNode(Word word) {
        return new WordHybridNode(word);
    }

    public static ProdHybridNode createProdHybridNode(ResElem elem, int cIndex,
            ArrayList<Word> allwords) {
        return new ProdHybridNode(elem, cIndex, allwords);
    }

    public void setParent(HybridNode parent) {
        this._parent = parent;
    }

    public void setChildren(ArrayList<HybridNode> children) {
        this._children = children;
    }

    public HybridNode getParent() {
        return this._parent;
    }

    public ArrayList<HybridNode> getChildren() {
        return this._children;
    }

    public String toString() {
        return this.toString(0);
    }

    protected abstract String toString(int indent);

    public abstract Word toWord();

}
