/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package hybridtree.core;

import hybridtree.data.Word;

import java.util.ArrayList;

public class ProdHybridNode extends HybridNode {
	
	private static final long serialVersionUID = 1562545963849024017L;
	private ResElem _elem;
    private int _cIndex; //the index as a child
    private ArrayList<Word> _allwords;

    protected ProdHybridNode(ResElem elem, int cIndex, ArrayList<Word> allwords) {
        this._elem = elem;
        this._cIndex = cIndex;
        this._allwords = allwords;
    }

    public ArrayList<Word> getAllWords() {
        return this._allwords;
    }

    public ResElem getElem() {
        return this._elem;
    }

    public boolean equals(Object o) {
        if (o instanceof ProdHybridNode) {
            ProdHybridNode pNode = (ProdHybridNode) o;
            boolean result = pNode._elem.equals(this._elem);
            result &= pNode._children.equals(this._children);
        }
        return false;
    }

    protected String toString(int indent) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < indent; i++) {
            sb.append('\t');
        }
        sb.append(this._elem.getGP().toLine());
        sb.append('[');
        sb.append(this._elem.getProb());
        sb.append(']');
        sb.append('\n');
        for (int i = 0; i < this._children.size(); i++) {
            HybridNode child = this._children.get(i);
            sb.append(child.toString(indent + 1));
        }
        return sb.toString();
    }

    public Word toWord() {
        return this._cIndex == 0 ? Word.CHILD_A : Word.CHILD_B;
    }
}
