/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.core;

/**
 * @author Wei LU
 * @version 1.0
 */

import java.util.Arrays;

public class ResElemList {

    private ResElem[] _elems;

    public static ResElemList _emptyList = null;

    @SuppressWarnings("unused")
	private ResElemList() {
        this._elems = new ResElem[0];
    }

    public ResElemList(int size) {
        if (size < 1) {
            throw new RuntimeException("Invalid ResElemList Size");
        }
        this._elems = new ResElem[size + 1];
        Arrays.fill(this._elems, ResElem._dummy);
    }

    public boolean add(ResElem res) {
        if (res.getProb() <= this._elems[1].getProb()) {
            return false;
        }

        int index = Arrays.binarySearch(this._elems, res);
        if ( index > 0) {
            return false;
        }

        index = -1 - index;
        for(int i = 1 ; i<index; i++){
            this._elems[i-1] = this._elems[i];
        }
        this._elems[index-1] = res;

        return true;
    }

    public int size() {
        return this._elems.length - 1;
    }

    public ResElem get(int index) {
        return this._elems[index + 1];
    }

    public ResElem getBest() {
        return this._elems[this._elems.length - 1];
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < this.size(); i++) {
            sb.append(this.get(i));
            sb.append('\n');
        }
        return sb.toString();
    }

    public boolean isEmpty(){
        return this.equals(_emptyList);
    }

}
