/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package hybridtree.core;

import hybridtree.data.Word;

public class WordHybridNode extends HybridNode {
	
	private static final long serialVersionUID = -848162183146814937L;
	
	private Word _word;

    protected WordHybridNode(Word word) {
        this._word = word;
    }

    public Word getWord() {
        return this._word;
    }

    public boolean equals(Object o) {
        if (o instanceof WordHybridNode) {
            WordHybridNode wNode = (WordHybridNode) o;
            boolean result = wNode.equals(_word);
            result &= wNode._children.equals(this._children);
        }
        return false;
    }

    protected String toString(int indent) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < indent; i++) {
            sb.append('\t');
        }
        sb.append(this._word.getWordStr());
        sb.append('\n');
        for (int i = 0; i < this._children.size(); i++) {
            HybridNode child = this._children.get(i);
            sb.append(child.toString(indent + 1));
        }
        return sb.toString();
    }

    public Word toWord() {
        return this._word;
    }

}
