/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.core.learn;

/**
 * @author LU, Wei
 * @version 1.0
 */

import java.util.ArrayList;

import hybridtree.core.*;
import hybridtree.data.InnerFeature;
import hybridtree.data.Production;
import hybridtree.data.TrainExample;
import hybridtree.data.Word;


public class NMBGTrainTable extends MBGTrainTable {

    /**
     * Constructor
     * @param example the Example
     * @param parameter the parameter
     */
    protected NMBGTrainTable(TrainExample example, Param parameter) {

        super(example, parameter);

        UBGTrainTable table = new UBGTrainTable(example, parameter);
        table.computeBeta();
        table.computeAlpha();

    }

    /**
     * Update the Word Parameter
     * @param prod Production
     * @param word Word
     * @param count double
     */
    protected void updateWordParam(Production prod, Word word1, Word word, double count) {
        if (Double.isInfinite(count)) {
            throw new RuntimeException("x");
        }

        if (count == 0) {
            return;
        }
        
        count *= this._weight;
        ArrayList<InnerFeature> innerFeatures = prod.getGProduction().getInnerFeatures();

        double ratio = 1.0 / (double) innerFeatures.size();
        for (int i = 0; i < innerFeatures.size(); i++) {

            InnerFeature innerFeature = innerFeatures.get(i);

            double subProb;

            subProb = count / this.computeOneWordProb(prod, word1, word) * this._parameter.getInnerFeatureProb(innerFeature, word1, word) * ratio;
            this._parameter.putCount(innerFeature, word1, word, subProb);

        }
    }

    protected double computeOneWordProb(Production prod, Word word1, Word word) {
        return this._parameter.getProb(prod.getGProduction(), word1, word);
    }

}
