/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package hybridtree.core.parse;

import hybridtree.core.*;
import hybridtree.data.*;

/**
 * @author Wei LU
 * @version 1.0
 */
public class MGParseTable extends BGParseTable{
    private Param _parameter_unigram;
    private Param _parameter_bigram;

    public MGParseTable(TestExample example, Param parameter_unigram, Param parameter_bigram) {
        super(example, parameter_unigram);
        this._parameter_unigram = parameter_unigram;
        this._parameter_bigram = parameter_bigram;
    }


    public double computeRuleTypeProb(GProduction prod, int patternType) {
        double prob1 = this._parameter_unigram.getPatternProb(prod, patternType);
        double prob2 = this._parameter_bigram.getPatternProb(prod, patternType);
        return (prob1+prob2)*.5;
    }


    protected double computeOneWordProb(GProduction prod, Word word1, Word word) {
        double prob1 = this._parameter_unigram.getProb(prod, word);
        double prob2 = this._parameter_bigram.getProb(prod, word1, word);
        return (prob1+prob2)*.5;
    }

}


