/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package hybridtree.core.perceptron;

/**
 * @author Wei LU
 * @version 1.0
 */
import hybridtree.util.Pair;

import java.io.Serializable;

public class FVPair extends Pair<PFeature, Double> implements Serializable, Comparable<FVPair> {

	private static final long serialVersionUID = 7989369053956948160L;

	/**
     * Constructor
     * @param feature PFeature
     * @param value double
     */
    public FVPair(PFeature feature, double value) {
        super(feature, value);
    }

    public boolean equals(Object o) {
        if (o instanceof FVPair) {
            FVPair f = (FVPair) o;
            return f.getFeature().equals(this.getFeature()) && f.getValue() == this.getValue();
        }
        return false;
    }

    /**
     * Get the PFeature
     * @return PFeature
     */
    public PFeature getFeature() {
        return (PFeature)super.getFirst();
    }

    /**
     * Get the cost for this pair
     * @return double
     */
    public double computeCost() {
        PFeature feature = this.getFeature();
        if (feature.isDisabled()) {
            return 0;
        }
        if(feature.appearedOnceOnly()){
            return 0;
        }
        return feature.getWeight() * this.getValue();
    }

    /**
     * Get the value of the PFeature
     * @return double
     */
    public double getValue() {
        return (Double)super.getSecond();
    }

    public int compareTo(FVPair fp) {
        int result = this.getFeature().compareTo(fp.getFeature());
        return result;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("FVPair:");
        sb.append('[');
        sb.append(this.getFeature());
        sb.append(',');
        sb.append(this.getValue());
        sb.append(']');
        return sb.toString();
    }
}
