/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package hybridtree.data;

import java.util.Vector;

/**
 * A context is a collection of features, (normally) associated with a particular Production
 */

public class Context {

    /**
     * the features that appeared in the context
     */
    private Vector<Feature> _features;

    /**
     * Extract the Context given a Production
     * @todo
     * @param prod the current Production
     * @return the Context
     */
    public static Context extractContext(Production prod){
        //todo...
        return null;
    }

    /**
     * Constructor
     * @param features a collection of features
     */
    public Context(Vector<Feature> features) {
        this._features = features;
    }

    /**
     * Add the feature to the feature set
     * @param feature Feature
     */
    public void addFeature(Feature feature) {
        //can be more efficient..
        for (int i = 0; i < this._features.size(); i++) {
            Feature currFeature = this._features.get(i);
            if (currFeature.compareTo(feature) > 0) {
                this._features.add(i, feature);
                return;
            }
        }
        this._features.add(feature);
    }

    /**
     * Get the feature by position
     * @param pos the position of the feature
     * @return the Feature at the specified position
     */
    public Feature getFeatureAt(int pos) {
        return this._features.get(pos);
    }

    /**
     * Get all the features
     * @return a collection of features
     */
    public Vector<Feature> getFeatures() {
        return this._features;
    }

    /**
     * Get the size of the Context, which is also the number of Features.
     * @return the size of the Context
     */
    public int size() {
        return this._features.size();
    }

    public boolean equals(Object o) {
        Vector<Feature> vFeatures = ((Context) o)._features;
        for (int i = 0; i < vFeatures.size(); i++) {
            Feature vf = vFeatures.get(i);
            Feature f = _features.get(i);
            if (!vf.equals(f)) {
                return false;
            }
        }
        return true;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append('<');
        for (int i = 0; i < this._features.size(); i++) {
            sb.append(this._features.get(i));
            sb.append('\n');
        }
        sb.append('>');
        return sb.toString();
    }

}

