/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package hybridtree.data;

/**
 * @author Wei LU
 * @version 1.0
 */

import java.io.Serializable;

public abstract class Feature implements Serializable, Comparable<Feature> {

	private static final long serialVersionUID = 3039678963844099043L;

	/**
     * the string representation of the feature
     */
    private String _f;

    /**
     * the feature index
     */
    private int _featureId;

    public Feature(String f) {
        this._f = f;
    }

    public void setFeatureId(int featureId) {
        this._featureId = featureId;
    }

    public int getFeatureId() {
        return this._featureId;
    }

    public String getFeatureStr() {
        return this._f;
    }

    public int compareTo(Feature feature) {
        if (this._featureId < feature.getFeatureId()) {
            return -1;
        } else if (this._featureId > feature.getFeatureId()) {
            return +1;
        } else {
            return 0;
        }
    }

    public boolean equals(Object o) {
        if (o instanceof Feature) {
            Feature feature = (Feature) o;
            return feature.getFeatureStr().equals(this._f);
        }
        return false;
    }

}
