/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.data;


/**
 * @author LU Wei
 * @version 1.0
 */
import java.io.Serializable;

public abstract class Node implements Serializable {

	private static final long serialVersionUID = -6127468945993260703L;

	/**
     * a string representation of the node
     */
    protected String _name;

    /**
     * the index of this node
     */
    private int _nodeIndex;

    /**
     * Constructor
     * @param name string representation of the node
     */
    protected Node(String name) {
        this._name = name;
    }

    /**
     * Set the string represnetation of the node
     * @param name the string representaion of the node
     */
    public void setName(String name) {
        this._name = name;
    }

    /**
     * Set the index for this node
     * @param nodeIndex the index of this node
     */
    public void setNodeIndex(int nodeIndex) {
        this._nodeIndex = nodeIndex;
    }

    /**
     * Get the index of this node
     * @return the index of this node
     */
    public int getNodeIndex() {
        return this._nodeIndex;
    }

    /**
     * Get the string representation of the node
     * @return the string representation of the node
     */
    public String getName() {
        return this._name;
    }

    public boolean equals(Object o) {
        if (o instanceof Node) {
            Node node = (Node) o;
            return this._name.equals(node._name) &&
                    this.getClass().equals(node.getClass());
        }
        return false;
    }

    public String toString() {
        return this._name;
    }

}
