/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package hybridtree.data;

/**
 * @author LU Wei
 * @version 1.0
 */

import java.util.ArrayList;

public class TrainExample extends Example {

	private static final long serialVersionUID = 3362729018352245492L;

	/**
     * the weight associated with the example, reflecting its importance
     */
    private double _weight = 1.0;

    /**
     * the flag indicating whether this example should be ignored
     */
    private boolean _ignoreFlag = false;

    /**
     * The constructor
     * @param exampleId the Example's id
     * @param nl the natural language sentence, in string form
     * @param mrl the meaning representation, in string form
     * @param productions all the productions
     */
    public TrainExample(int exampleId, String nl, ArrayList<Word> words,
                        String mrl, ArrayList<Production> productions) {
        super(exampleId, nl, words, mrl, productions);
    }

    /**
     * Convert this TrainExample to a TestExample
     * @return TestExample
     */
    public TestExample toTestExample() {
        return new TestExample(this._exampleId, this._nl, this._words,
                               this._mrl, this._productions);
    }

    /**
     * Set the weight for this Example
     * @param weight the weight to be assigned
     */
    public void setWeight(double weight) {
        this._weight = weight;
    }

    /**
     * Get the weight of this Example
     * @return double
     */
    public double getWeight() {
        return this._weight;
    }

    /**
     * Get the root Production of this Example
     * @return the root Production
     */
    public Production getRootProduction() {
        Production root = (Production)this._productions.get(0);
        return root;
    }

    /**
     * Mark this example as ignored
     */
    public void setIgnore() {
        this._ignoreFlag = true;
    }

    /**
     * Check whether this train example should be ignored from training
     * @return boolean
     */
    public boolean ignore() {
        return this._ignoreFlag;
    }

    /**
     * Get all the Productions of this Example
     * @return all the Productions
     */
    public ArrayList<Production> getProductions() {
        return this._productions;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();

        sb.append("instance:" + this._exampleId + "\n");
        sb.append("en:" + this._nl + "\n");
        sb.append("mrl:" + this._mrl + "\n");
        sb.append("MRL:\n" + this.getRootProduction());
        sb.append("\n");

        return sb.toString();
    }

}
