/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.data;

/**
 * @author LU, Wei
 * @version 1.0
 */
import java.io.*;

public class Word implements Serializable {

	private static final long serialVersionUID = -4378649208419502571L;

	/**
     * the string representation of the Word
     */
    private String _word;

    /**
     * the index of this Word
     */
    private int _wordId;

    /**
     * The Begin symbol
     */
    public static Word BEGIN = new Word("#BEGIN#", 0);

    /**
     * The End symbol
     */
    public static Word END = new Word("#END#", 1);

    /**
     * The symbol for the first child
     */
    public static Word CHILD_A = new Word("#CHILD_A#", 2);

    /**
     * The symbol for the second child
     */
    public static Word CHILD_B = new Word("#CHILD_B#", 3);

    /**
     * The symbol for the unknown word
     */
    public static Word UNKNOWN = new Word("#UNKNOWN#", 4);

    public static final int BEGIN_ID = BEGIN._wordId;
    public static final int END_ID = END._wordId;
    public static final int CHILD_A_ID = CHILD_A._wordId;
    public static final int CHILD_B_ID = CHILD_B._wordId;
    public static final int UNKNOWN_ID = UNKNOWN._wordId;

    private Word(String word, int wordId) {
        this._word = word;
        this._wordId = wordId;
    }

    /**
     * Constructor
     * @param word the string form of this Word
     */
    protected Word(String word) {
        this._word = word;
    }

    /**
     * Set the word index
     * @param wordIndex the word index
     */
    public void setWordId(int wordId) {
        this._wordId = wordId;
    }

    /**
     * Get the word index
     * @return the word index
     */
    public int getWordId() {
        return this._wordId;
    }

    /**
     * Get the word string
     * @return the word string
     */
    public String getWordStr() {
        return this._word;
    }

    public boolean equals(Object o) {
        if (o instanceof Word) {
            Word w = (Word) o;
            return w._word.equals(_word);
        }
        return false;
    }

    public String toString() {
        return this._word + "|" + this._wordId;
    }

}
