/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *  
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.data.reader;

/**
 *
 * @author LU Wei
 * @version 1.0
 */
import java.util.*;

import hybridtree.data.*;

public class Binarizer {
	
    public static void main(String args[]) throws Exception {
        Model model = ModelReader.readAtis();
        
        doIt(model);
    }

    public static void doIt(Model model) {

        Model newModel = new Model();

        HashMap<String, String> table = new HashMap<String, String>();

        ArrayList<Example> train_examples = model.getTrainExamples();

        for (int i = 0; i < train_examples.size(); i++) {
            TrainExample ex = (TrainExample) train_examples.get(i);
            Production root = ex.getRootProduction();

            if (ex.getExampleId() < 0) {
                continue;
            }

            root = standardize(root, model);

            Production newRoot = binarize(root, table, newModel);

            System.out.println("id:" + ex.getExampleId());
            System.out.print("nl:");
            System.out.println(ex.getNL());
            System.out.println("mrl:true");
            System.out.println("productions:");

            ArrayList<Production> allNewProds = newRoot.getAllChildren();
            for (int j = 0; j < allNewProds.size(); j++) {
                System.out.println(allNewProds.get(j).toLine());
            }
            System.out.println();
        }
    }


    private static Production standardize(Production prod, Model model) {
        ArrayList<Production> children = prod.getChildren();

        int n = children.size();

        int index = prod.toLine().lastIndexOf("\"");

        if (n != 0 && index != -1) {
            NonterminalNode lhs = prod.getLHS();

            ArrayList<Node> newRHS1 = new ArrayList<Node>();
            ArrayList<Node> newRHS2 = new ArrayList<Node>();

            NonterminalNode newNT = model.createNonterminalNode(lhs.getName() +
                    "0");

            ArrayList<Node> rhss = prod.getRHS();

            newRHS2.add(newNT);
            for (int i = 0; i < rhss.size(); i++) {
                Node rhs = rhss.get(i);
                if (rhs instanceof NonterminalNode) {
                    newRHS2.add(rhs);
                } else {
                    newRHS1.add(rhs);
                }
            }

            GProduction gp1 = model.createGProduction(lhs, newRHS2);
            GProduction gp2 = model.createGProduction(newNT, newRHS1);

            Production prod1 = new Production(gp1);
            Production prod2 = new Production(gp2);

            ArrayList<Production> newChildren = new ArrayList<Production>();
            newChildren.add(prod2);

            prod2.setParent(prod1);

            for (int i = 0; i < children.size(); i++) {
                Production child = children.get(i);

                Production newChild = standardize(child, model);
                newChild.setParent(prod1);
                newChildren.add(newChild);
            }
            prod1.setChildren(newChildren);

            return prod1;
        }

        ArrayList<Production> newChildren = new ArrayList<Production>();

        for (int i = 0; i < children.size(); i++) {
            Production child = children.get(i);
            Production newChild = standardize(child, model);

            newChildren.add(newChild);
            newChild.setParent(prod);
        }

        prod.setChildren(newChildren);

        return prod;
    }

    private static Production binarize(Production prod, HashMap<String, String> table,
                                       Model model) {

        ArrayList<Production> children = prod.getChildren();

        int n = children.size();

        if (n <= 2) {
            ArrayList<Production> newChildren = new ArrayList<Production>();
            for (int i = 0; i < n; i++) {
                Production child = children.get(i);
                Production newChild = binarize(child, table, model);

                newChildren.add(newChild);
                newChild.setParent(prod);
            }

            prod.setChildren(newChildren);

            return prod;

        } else {
            StringBuffer keyB = new StringBuffer();

            NonterminalNode lhs = prod.getLHS();

            ArrayList<Node> rhss = prod.getRHS();

            ArrayList<Node> newRHS1 = new ArrayList<Node>();
            ArrayList<Node> newRHS2 = new ArrayList<Node>();

            keyB.append(lhs);
            keyB.append('\n');

            int rhsNTcount = 0;

            for (int i = 0; i < rhss.size(); i++) {
                Node rhs = rhss.get(i);
                if (rhs instanceof NonterminalNode && rhsNTcount < 2) {
                    rhsNTcount++;
                    keyB.append(rhs);
                    keyB.append('\n');
                } else {
                }
            }

            NonterminalNode newNT = model.createNonterminalNode(nextSymbol(lhs.
                    getName(), table, keyB.toString()));

            for (int i = 0; i < rhss.size(); i++) {
                Node rhs = rhss.get(i);
                if (rhs instanceof NonterminalNode && newRHS2.size() == 0) {
                    newRHS1.add(newNT);
                    newRHS2.add(rhs);
                } else if (rhs instanceof NonterminalNode &&
                           newRHS2.size() == 1) {
                    newRHS2.add(rhs);
                } else {
                    newRHS1.add(rhs);
                    keyB.append(rhs);
                    keyB.append('\n');
                }
            }

            GProduction gp1 = model.createGProduction(lhs, newRHS1);
            GProduction gp2 = model.createGProduction(newNT, newRHS2);

            Production prod1 = new Production(gp1);
            Production prod2 = new Production(gp2);

            ArrayList<Production> children1 = new ArrayList<Production>();

            children1.add(prod2);
            prod2.setParent(prod1);
            for (int i = 2; i < children.size(); i++) {
                Production child = children.get(i);

                Production newChild = binarize(child, table, model);
                children1.add(newChild);
                newChild.setParent(prod1);
            }
            prod1.setChildren(children1);

            ArrayList<Production> children2 = new ArrayList<Production>();
            for (int i = 0; i < 2; i++) {
                Production child = children.get(i);

                Production newChild = binarize(child, table, model);
                children2.add(newChild);
                newChild.setParent(prod2);
            }
            prod2.setChildren(children2);

            return binarize(prod1, table, model);

        }

    }

    private static HashMap<String,
            Integer> symbolTable = new HashMap<String, Integer>();

    private static String nextSymbol(String name, HashMap<String,
                                     String> table, String key) {
        if (table.containsKey(key)) {
            return table.get(key);
        }

        String elem;
        if (!symbolTable.containsKey(name)) {
            symbolTable.put(name, 2);
            elem = name + "1";
        } else {
            int val = symbolTable.get(name);
            symbolTable.put(name, val + 1);
            elem = name + "" + val;
        }

        table.put(key, elem);
        return elem;
    }

}
