/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package hybridtree.data.reader;

/**
 * @author LU Wei
 * @version 1.0
 */
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;

import java.util.StringTokenizer;
import java.util.ArrayList;

import hybridtree.data.GProduction;
import hybridtree.data.Model;
import hybridtree.data.Node;
import hybridtree.data.NonterminalNode;
import hybridtree.data.Production;
import hybridtree.data.TestExample;
import hybridtree.data.TrainExample;
import hybridtree.data.Word;
import hybridtree.util.Utility;



public class ModelReader {


    public static Model readAtis() throws IOException
             {

        String filename_train = "data/atis-funql/atis.train.corpus";

        FileInputStream infile_train = new FileInputStream(filename_train);
        BufferedReader stdin_train = new BufferedReader(new InputStreamReader(
                infile_train));

        Model model = new Model();

        //set the root NonterminalNode
        model.createNonterminalNode("*n:SS");

        String line;
        while ((line = stdin_train.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());

                String nl = stdin_train.readLine();
                if (!nl.startsWith("nl:")) {
                    throw new ModelReaderException("Expected NL missing:\n\t" +
                            nl);
                } else {
                    nl = nl.substring(3).trim();
                }
                String mrl = stdin_train.readLine();
                if (!mrl.startsWith("mrl:")) {
                    throw new ModelReaderException("Expected MRL missing:\n\t" +
                            mrl);
                } else {
                    mrl = mrl.substring(4).trim();
                }
                ArrayList<String> prodStrs = new ArrayList<String>();
                if (stdin_train.readLine().startsWith("productions:")) {
                    while (!(line = stdin_train.readLine()).trim().equals("")) {
                        prodStrs.add(line);
                    }
                }
                ArrayList<Production>
                        productions = parseProductions(prodStrs, model);

                StringTokenizer st = new StringTokenizer(nl);
                ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                while (st.hasMoreTokens()) {
                    String w = st.nextToken();
                    Word word = model.createWord(w);
                    words.add(word);
                }

                TrainExample ex = new TrainExample(id, nl, words, mrl,
                        productions);
                ex.setWeight(1.0);

                model.addTrainExample(ex);
            }
        }

        infile_train.close();

        //adding initial lexicon
        String filename_init = "data/atis-funql/atis.init.corpus";

        FileInputStream infile_init = new FileInputStream(filename_init);
        BufferedReader stdin_init = new BufferedReader(new InputStreamReader(
                infile_init));

        while ((line = stdin_init.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());
                String nl = stdin_init.readLine();
                if (!nl.startsWith("nl:")) {
                    throw new ModelReaderException("Expected NL missing:\n\t" +
                            nl);
                } else {
                    nl = nl.substring(3).trim();
                }
                String mrl = stdin_init.readLine();
                if (!mrl.startsWith("mrl:")) {
                    throw new ModelReaderException("Expected MRL missing:\n\t" +
                            mrl);
                } else {
                    mrl = mrl.substring(4).trim();
                }
                ArrayList<String> prodStrs = new ArrayList<String>();
                if (stdin_init.readLine().startsWith("productions:")) {
                    while (!(line = stdin_init.readLine()).trim().equals("")) {
                        prodStrs.add(line);
                    }
                }
                ArrayList<Production>
                        productions = parseProductions(prodStrs, model);

                StringTokenizer st = new StringTokenizer(nl);
                ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                while (st.hasMoreTokens()) {
                    String w = st.nextToken();
                    Word word = model.createWord(w);
                    words.add(word);
                }

                TrainExample ex = new TrainExample(id, nl, words, mrl,
                        productions);
                ex.setWeight(1000.0);

                model.addTrainExample(ex);
            }
        }

        stdin_init.close();

        String filename_test = "data/atis-funql/atis.test.corpus";

        //adding test instances..
        FileInputStream infile_test = new FileInputStream(filename_test);
        BufferedReader stdin_test = new BufferedReader(new InputStreamReader(
                infile_test));

        while ((line = stdin_test.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());
                String nl = stdin_test.readLine();
                if (!nl.startsWith("nl:")) {
                    throw new ModelReaderException(
                            "Expected NL missing:\n\t" + nl);
                } else {
                    nl = nl.substring(3).trim();
                }
                String mrl = stdin_test.readLine();
                if (!mrl.startsWith("mrl:")) {
                    throw new ModelReaderException(
                            "Expected MRL missing:\n\t" + mrl);
                } else {
                    mrl = mrl.substring(4).trim();
                }
                ArrayList<String> prodStrs = new ArrayList<String>();
                if (stdin_test.readLine().startsWith("productions:")) {
                    while (!(line = stdin_test.readLine()).trim().equals("")) {
                        prodStrs.add(line);
                    }
                }
                ArrayList<Production>
                        productions = parseProductions(prodStrs, model);

                StringTokenizer st = new StringTokenizer(nl);
                ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                while (st.hasMoreTokens()) {
                    String w = st.nextToken();
                    Word word = model.attemptCreateWord(w);
                    words.add(word);
                }

                TestExample ex = new TestExample(id, nl, words, mrl,
                                                 productions);

                model.addTestExample(ex);
            }
        }

        infile_test.close();

        return model;
    }


    public static Model readGeoqueryFunql(int run, int fold) throws IOException {
        return readGeoqueryFunql(run, fold, "en", 880, 792);
    }


    public static Model readAtis(int run, int fold) throws IOException {
        return readAtis(run, fold, "en", 4978, 3980);
    }

    public static Model readAtis(int run, int fold, String lang,
                                 int numTotalExamples,
                                 int numTrainExamples
            ) throws
            IOException {

        String filename_train = "data/atis-funql/split-" + numTotalExamples +
                                "/run-" + run +
                                "/fold-" + fold + "/train-N" + numTrainExamples;

        FileInputStream infile_train = new FileInputStream(filename_train);
        BufferedReader stdin_train = new BufferedReader(new InputStreamReader(
                infile_train));

        String line_train;
        ArrayList<Integer> selectedIds_train = new ArrayList<Integer>();
        while ((line_train = stdin_train.readLine()) != null) {
            int id = Integer.parseInt(line_train);
            selectedIds_train.add(new Integer(id));
        }

        String filename_test = "data/atis-funql/split-4978/run-" + run +
                               "/fold-" + fold + "/test";

        FileInputStream infile_test = new FileInputStream(filename_test);
        BufferedReader stdin_test = new BufferedReader(new InputStreamReader(
                infile_test));

        String line_test;
        ArrayList<Integer> selectedIds_test = new ArrayList<Integer>();
        while ((line_test = stdin_test.readLine()) != null) {
            int id = Integer.parseInt(line_test);
            selectedIds_test.add(new Integer(id));
        }

        Model model = new Model();

        //set the root NonterminalNode
        model.createNonterminalNode("*n:SS");

        FileInputStream infile;
        BufferedReader stdin;

        //adding training instances
        infile = new FileInputStream("data/atis-funql/atisFunql-" + lang +
                                     ".corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        String line;
        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());

                if (selectedIds_train.contains(id)) {
                    String nl = stdin.readLine();
                    if (!nl.startsWith("nl:")) {
                        throw new ModelReaderException(
                                "Expected NL missing:\n\t" + nl);
                    } else {
                        nl = nl.substring(3).trim();
                    }
                    String mrl = stdin.readLine();
                    if (!mrl.startsWith("mrl:")) {
                        throw new ModelReaderException(
                                "Expected MRL missing:\n\t" + mrl);
                    } else {
                        mrl = mrl.substring(4).trim();
                    }
                    ArrayList<String> prodStrs = new ArrayList<String>();
                    if (stdin.readLine().startsWith("productions:")) {
                        while (!(line = stdin.readLine()).trim().equals("")) {
                            prodStrs.add(line);
                        }
                    }
                    ArrayList<Production>
                            productions = parseProductions(prodStrs, model);

                    StringTokenizer st = new StringTokenizer(nl);
                    ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                    while (st.hasMoreTokens()) {
                        String w = st.nextToken();
                        Word word = model.createWord(w);
                        words.add(word);
                    }

                    TrainExample ex = new TrainExample(id, nl, words, mrl,
                            productions);
                    ex.setWeight(1.0);

                    model.addTrainExample(ex);
                } else if (selectedIds_test.contains(id)) {
                    //do not do anything...
                } else {
                    //System.err.println("[WARNING] Example " + id + " excluded from both training and testing!");
                }
            }

        }
        infile.close();

        //adding initial lexicon
        infile = new FileInputStream("data/atis-funql/atisFunql-" + lang +
                                     ".init.corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());
                String nl = stdin.readLine();
                if (!nl.startsWith("nl:")) {
                    throw new ModelReaderException("Expected NL missing:\n\t" +
                            nl);
                } else {
                    nl = nl.substring(3).trim();
                }
                String mrl = stdin.readLine();
                if (!mrl.startsWith("mrl:")) {
                    throw new ModelReaderException("Expected MRL missing:\n\t" +
                            mrl);
                } else {
                    mrl = mrl.substring(4).trim();
                }
                ArrayList<String> prodStrs = new ArrayList<String>();
                if (stdin.readLine().startsWith("productions:")) {
                    while (!(line = stdin.readLine()).trim().equals("")) {
                        prodStrs.add(line);
                    }
                }
                ArrayList<Production>
                        productions = parseProductions(prodStrs, model);

                //mark them as constants...
                for (int i = 0; i < productions.size(); i++) {
                    GProduction gp = productions.get(i).getGProduction();
                    gp.setConstant();
                }

                StringTokenizer st = new StringTokenizer(nl);
                ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                while (st.hasMoreTokens()) {
                    String w = st.nextToken();
                    Word word = model.createWord(w);
                    words.add(word);
                }

                TrainExample ex = new TrainExample(id, nl, words, mrl,
                        productions);
                ex.setWeight(1000.0);

                model.addTrainExample(ex);
            }
        }
        infile.close();

        //adding test instances..
        infile = new FileInputStream("data/atis-funql/atisFunql-" + lang +
                                     ".corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());
                if (selectedIds_test.contains(id)) {
                    String nl = stdin.readLine();
                    if (!nl.startsWith("nl:")) {
                        throw new ModelReaderException(
                                "Expected NL missing:\n\t" + nl);
                    } else {
                        nl = nl.substring(3).trim();
                    }
                    String mrl = stdin.readLine();
                    if (!mrl.startsWith("mrl:")) {
                        throw new ModelReaderException(
                                "Expected MRL missing:\n\t" + mrl);
                    } else {
                        mrl = mrl.substring(4).trim();
                    }
                    ArrayList<String> prodStrs = new ArrayList<String>();
                    if (stdin.readLine().startsWith("productions:")) {
                        while (!(line = stdin.readLine()).trim().equals("")) {
                            prodStrs.add(line);
                        }
                    }
                    ArrayList<Production>
                            productions = parseProductions(prodStrs, model);

                    StringTokenizer st = new StringTokenizer(nl);
                    ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                    while (st.hasMoreTokens()) {
                        String w = st.nextToken();
                        Word word = model.attemptCreateWord(w);
                        words.add(word);
                    }

                    TestExample ex = new TestExample(id, nl, words, mrl,
                            productions);

                    model.addTestExample(ex);

                    if (selectedIds_train.contains(id)) {
                        System.err.println("[WARNING] Example " + id +
                                           " included in both training and testing!");
                    }

                } else if (selectedIds_train.contains(id)) {
                    //do not do anything...
                } else {
                    //do not do anything...
                }
            }

        }
        infile.close();

        return model;

    }

    /**
     * Read the Geoquery Functional Query Corpus
     * @param run the run Id
     * @param fold the fold Id
     * @return Model
     * @throws IOException
     */
    public static Model readGeoqueryFunql(int run, int fold, String lang,
                                          int numTotalExamples,
                                          int numTrainExamples) throws
            IOException {

        String filename_train = "data/geo-funql/split-" + numTotalExamples +
                                "/run-" + run + "/fold-" + fold + "/train-N" +
                                numTrainExamples;

        FileInputStream infile_train = new FileInputStream(filename_train);
        BufferedReader stdin_train = new BufferedReader(new InputStreamReader(
                infile_train));

        String line_train;
        ArrayList<Integer> selectedIds_train = new ArrayList<Integer>();
        while ((line_train = stdin_train.readLine()) != null) {
            int id = Integer.parseInt(line_train);
            selectedIds_train.add(new Integer(id));
        }

        String filename_test = "data/geo-funql/split-" + numTotalExamples +
                               "/run-" + run + "/fold-" + fold + "/test";

        FileInputStream infile_test = new FileInputStream(filename_test);
        BufferedReader stdin_test = new BufferedReader(new InputStreamReader(
                infile_test));

        String line_test;
        ArrayList<Integer> selectedIds_test = new ArrayList<Integer>();
        while ((line_test = stdin_test.readLine()) != null) {
            int id = Integer.parseInt(line_test);
            selectedIds_test.add(new Integer(id));
        }

        Model model = new Model();

        //set the root NonterminalNode
        model.createNonterminalNode("*n:Query");

        FileInputStream infile;
        BufferedReader stdin;

        //adding training instances
        infile = new FileInputStream("data/geo-funql/geoFunql-" + lang +
                                     ".corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        String line;
        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());

                if (selectedIds_train.contains(id)) {
                    String nl = stdin.readLine();
                    if (!nl.startsWith("nl:")) {
                        throw new ModelReaderException(
                                "Expected NL missing:\n\t" + nl);
                    } else {
                        nl = nl.substring(3).trim();
                    }
                    String mrl = stdin.readLine();
                    if (!mrl.startsWith("mrl:")) {
                        throw new ModelReaderException(
                                "Expected MRL missing:\n\t" + mrl);
                    } else {
                        mrl = mrl.substring(4).trim();
                    }
                    ArrayList<String> prodStrs = new ArrayList<String>();
                    if (stdin.readLine().startsWith("productions:")) {
                        while (!(line = stdin.readLine()).trim().equals("")) {
                            prodStrs.add(line);
                        }
                    }
                    ArrayList<Production>
                            productions = parseProductions(prodStrs, model);

                    StringTokenizer st = new StringTokenizer(nl);
                    ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                    while (st.hasMoreTokens()) {
                        String w = st.nextToken();
                        Word word = model.createWord(w);
                        words.add(word);
                    }

                    TrainExample ex = new TrainExample(id, nl, words, mrl,
                            productions);
                    ex.setWeight(1.0);

                    model.addTrainExample(ex);
                } else if (selectedIds_test.contains(id)) {
                    //do not do anything...
                } else {
                    //System.err.println("[WARNING] Example " + id + " excluded from both training and testing!");
                }
            }

        }
        infile.close();

        //adding initial lexicon
        infile = new FileInputStream("data/geo-funql/geoFunql-" + lang +
                                     ".init.corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());
                String nl = stdin.readLine();
                if (!nl.startsWith("nl:")) {
                    throw new ModelReaderException("Expected NL missing:\n\t" +
                            nl);
                } else {
                    nl = nl.substring(3).trim();
                }
                String mrl = stdin.readLine();
                if (!mrl.startsWith("mrl:")) {
                    throw new ModelReaderException("Expected MRL missing:\n\t" +
                            mrl);
                } else {
                    mrl = mrl.substring(4).trim();
                }
                ArrayList<String> prodStrs = new ArrayList<String>();
                if (stdin.readLine().startsWith("productions:")) {
                    while (!(line = stdin.readLine()).trim().equals("")) {
                        prodStrs.add(line);
                    }
                }
                ArrayList<Production>
                        productions = parseProductions(prodStrs, model);

                //mark them as constants...
                for (int i = 0; i < productions.size(); i++) {
                    GProduction gp = productions.get(i).getGProduction();
                    gp.setConstant();
                }

                StringTokenizer st = new StringTokenizer(nl);
                ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                while (st.hasMoreTokens()) {
                    String w = st.nextToken();
                    Word word = model.createWord(w);
                    words.add(word);
                }

                TrainExample ex = new TrainExample(id, nl, words, mrl,
                        productions);
                ex.setWeight(1000.0);

                model.addTrainExample(ex);

            }

        }
        infile.close();

        //adding test instances..
        infile = new FileInputStream("data/geo-funql/geoFunql-" + lang +
                                     ".corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());
                if (selectedIds_test.contains(id)) {
                    String nl = stdin.readLine();
                    if (!nl.startsWith("nl:")) {
                        throw new ModelReaderException(
                                "Expected NL missing:\n\t" + nl);
                    } else {
                        nl = nl.substring(3).trim();
                    }
                    String mrl = stdin.readLine();
                    if (!mrl.startsWith("mrl:")) {
                        throw new ModelReaderException(
                                "Expected MRL missing:\n\t" + mrl);
                    } else {
                        mrl = mrl.substring(4).trim();
                    }
                    ArrayList<String> prodStrs = new ArrayList<String>();
                    if (stdin.readLine().startsWith("productions:")) {
                        while (!(line = stdin.readLine()).trim().equals("")) {
                            prodStrs.add(line);
                        }
                    }
                    ArrayList<Production>
                            productions = parseProductions(prodStrs, model);

                    StringTokenizer st = new StringTokenizer(nl);
                    ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                    while (st.hasMoreTokens()) {
                        String w = st.nextToken();
                        Word word = model.attemptCreateWord(w);
                        words.add(word);
                    }

                    TestExample ex = new TestExample(id, nl, words, mrl,
                            productions);

                    model.addTestExample(ex);

                    if (selectedIds_train.contains(id)) {
                        System.err.println("[WARNING] Example " + id +
                                           " included in both training and testing!");
                    }

                } else if (selectedIds_train.contains(id)) {
                    //do not do anything...
                } else {
                    //do not do anything...
                }
            }

        }
        infile.close();

        return model;
    }

    public static Model readRobocupClang(int run, int fold) throws IOException {
        return readRobocupClang(run, fold, 300, 270);
    }

    /**
     * Read the Robocup Clang Corpus
     * @param run the run Id
     * @param fold the fold Id
     * @return Model
     * @throws IOException
     */
    public static Model readRobocupClang(int run, int fold,
                                         int numTotalExamples,
                                         int numTrainExamples) throws
            IOException {

        String filename_train = "data/robocup-clang/split-" + numTotalExamples +
                                "/run-" + run + "/fold-" + fold + "/train-N" +
                                numTrainExamples;

        FileInputStream infile_train = new FileInputStream(filename_train);
        BufferedReader stdin_train = new BufferedReader(new InputStreamReader(
                infile_train));

        String line_train;
        ArrayList<Integer> selectedIds_train = new ArrayList<Integer>();
        while ((line_train = stdin_train.readLine()) != null) {
            int id = Integer.parseInt(line_train);
            selectedIds_train.add(new Integer(id));
        }

        String filename_test = "data/robocup-clang/split-" + numTotalExamples +
                               "/run-" + run + "/fold-" + fold + "/test";

        FileInputStream infile_test = new FileInputStream(filename_test);
        BufferedReader stdin_test = new BufferedReader(new InputStreamReader(
                infile_test));

        String line_test;
        ArrayList<Integer> selectedIds_test = new ArrayList<Integer>();
        while ((line_test = stdin_test.readLine()) != null) {
            int id = Integer.parseInt(line_test);
            selectedIds_test.add(new Integer(id));
        }

        Model model = new Model();

        //set the root nonterminal.
        model.createNonterminalNode("*n:Statement");

        String filename_comm = "data/robocup-clang/commutative";
        FileInputStream infile_comm = new FileInputStream(filename_comm);
        BufferedReader stdin_comm = new BufferedReader(new InputStreamReader(
                infile_comm));

        String line_comm;
        while ((line_comm = stdin_comm.readLine()) != null) {
            GProduction gp = ModelReader.parseGProduction(line_comm, model);
            gp.setCommutative();
        }

        FileInputStream infile;
        BufferedReader stdin;
        String line;

        //the training instances
        infile = new FileInputStream("data/robocup-clang/robocupClang.corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());

                if (selectedIds_train.contains(new Integer(id))) {

                    String nl = stdin.readLine();
                    if (!nl.startsWith("nl:")) {
                        throw new ModelReaderException(
                                "Expected NL missing:\n\t" + nl);
                    } else {
                        nl = nl.substring(3).trim();
                    }
                    String mrl = stdin.readLine();
                    if (!mrl.startsWith("mrl:")) {
                        throw new ModelReaderException(
                                "Expected MRL missing:\n\t" + mrl);
                    } else {
                        mrl = mrl.substring(4).trim();
                    }

                    ArrayList<String> prodStrs = new ArrayList<String>();
                    if (stdin.readLine().startsWith("productions:")) {
                        while (!(line = stdin.readLine()).trim().equals("")) {
                            prodStrs.add(line);
                        }
                    }
                    ArrayList<Production>
                            productions = parseProductions(prodStrs, model);

                    StringTokenizer st = new StringTokenizer(nl);
                    ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                    while (st.hasMoreTokens()) {
                        String w = st.nextToken();
                        Word word = model.createWord(w);
                        words.add(word);
                    }

                    TrainExample ex = new TrainExample(id, nl, words, mrl,
                            productions);
                    ex.setWeight(1.0);

                    model.addTrainExample(ex);
                } else if (selectedIds_test.contains(id)) {
                    //do not do anything...
                } else {
                    System.err.println("[WARNING] Example " + id +
                                       " excluded from both training and testing!");
                }
            }
        }
        infile.close();

        infile = new FileInputStream(
                "data/robocup-clang/robocupClang.init.corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());
                String nl = stdin.readLine();
                if (!nl.startsWith("nl:")) {
                    throw new ModelReaderException("Expected NL missing:\n\t" +
                            nl);
                } else {
                    nl = nl.substring(3).trim();
                }
                String mrl = stdin.readLine();
                if (!mrl.startsWith("mrl:")) {
                    throw new ModelReaderException("Expected MRL missing:\n\t" +
                            mrl);
                } else {
                    mrl = mrl.substring(4).trim();
                }

                ArrayList<String> prodStrs = new ArrayList<String>();
                if (stdin.readLine().startsWith("productions:")) {
                    while (!(line = stdin.readLine()).trim().equals("")) {
                        prodStrs.add(line);
                    }
                }
                ArrayList<Production>
                        productions = parseProductions(prodStrs, model);

                //mark them as constants...
                for (int i = 0; i < productions.size(); i++) {
                    GProduction gp = productions.get(i).getGProduction();
                    gp.setConstant();
                }

                StringTokenizer st = new StringTokenizer(nl);
                ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                while (st.hasMoreTokens()) {
                    String w = st.nextToken();
                    Word word = model.createWord(w);
                    words.add(word);
                }

                TrainExample ex = new TrainExample(id, nl, words, mrl,
                        productions);
                ex.setWeight(1000.0);

                model.addTrainExample(ex);

            }

        }
        infile.close();

        //the testing instances
        infile = new FileInputStream("data/robocup-clang/robocupClang.corpus");
        stdin = new BufferedReader(new InputStreamReader(infile));

        while ((line = stdin.readLine()) != null) {

            if (line.startsWith("id:")) {
                int id = Integer.parseInt(line.substring(3).trim());

                if (selectedIds_test.contains(new Integer(id))) {

                    String nl = stdin.readLine();
                    if (!nl.startsWith("nl:")) {
                        throw new ModelReaderException(
                                "Expected NL missing:\n\t" + nl);
                    } else {
                        nl = nl.substring(3).trim();
                    }
                    String mrl = stdin.readLine();
                    if (!mrl.startsWith("mrl:")) {
                        throw new ModelReaderException(
                                "Expected MRL missing:\n\t" + mrl);
                    } else {
                        mrl = mrl.substring(4).trim();
                    }

                    ArrayList<String> prodStrs = new ArrayList<String>();
                    if (stdin.readLine().startsWith("productions:")) {
                        while (!(line = stdin.readLine()).trim().equals("")) {
                            prodStrs.add(line);
                        }
                    }
                    ArrayList<Production>
                            productions = parseProductions(prodStrs, model);

                    StringTokenizer st = new StringTokenizer(nl);
                    ArrayList<Word> words = new ArrayList<Word>(st.countTokens());
                    while (st.hasMoreTokens()) {
                        String w = st.nextToken();
                        Word word = model.attemptCreateWord(w);
                        words.add(word);
                    }

                    TestExample ex = new TestExample(id, nl, words, mrl,
                            productions);

                    model.addTestExample(ex);

                    if (selectedIds_train.contains(id)) {
                        System.err.println("[WARNING] Example " + id +
                                           " included in both training and testing!");
                    }

                } else if (selectedIds_train.contains(id)) {
                    //do not do anything...
                } else {
                    //do not do anything...
                }

            }
        }
        infile.close();

        return model;
    }

    /**
     * Parse a collection of strings to a collection of Productions
     * @param prodStrs a collection of strings
     * @return a collection of Productions
     */
    public static ArrayList<Production> parseProductions(ArrayList<String> prodStrs,
            Model model) {

        ArrayList<Production> productions = new ArrayList<Production>();

        ArrayList<Production> prodArrayList = new ArrayList<Production>();
        for (int i = 0; i < prodStrs.size(); i++) {
            String product = prodStrs.get(i);
            GProduction gp = parseGProduction(product, model);
            Production prod = new Production(gp);
            prod.setProdSeq(i); //added. to record the sequance number
            prodArrayList.add(prod);
        }

        for (int i = 0; i < prodArrayList.size(); i++) {
            Production prod = prodArrayList.get(i);
            ArrayList<Node> RHS_nodes = prod.getRHS();
            ArrayList<Production> children = new ArrayList<Production>();
            int nonterminalCount = 0;
            for (int j = 0; j < RHS_nodes.size(); j++) {
                Node RHS_node = (Node) RHS_nodes.get(j);
                if (RHS_node instanceof NonterminalNode) {
                    NonterminalNode nNode = (NonterminalNode) RHS_node;
                    Production p = findReferringProduction(prodArrayList, i,
                            nonterminalCount);
                    nonterminalCount++;
                    if (!p.getLHS().getName().equals(nNode.getName())) {
                        System.err.println("Error![" + p.getLHS().getName() +
                                           "] : [" + nNode.getName() + "]" + i);
                        for (int k = 0; k < prodStrs.size(); k++) {
                            System.err.println(prodStrs.get(k));
                        }
                        System.err.println();
                        System.exit(1);
                    }
                    p.setLHS(nNode);
                    children.add(p);
                    p.setParent(prod);
                }
            }
            prod.setChildren(children);
            productions.add(prod);
        }
        return productions;

    }

    private static Production findReferringProduction(ArrayList<Production>
            prodcts, int startProductionIndex, int nonterminalCount) {

        int count = nonterminalCount;
        for (int i = startProductionIndex + 1; i < prodcts.size(); i++) {
            Production prod = prodcts.get(i);
            if (count == 0) {
                return prod;
            }

            count--;
            count += prod.countRHSNonterminals();
        }

        return null;
    }

    /**
     * Parse a string representation of a production to a Production
     * @param prodStr a string to be parsed
     * @return a resulting Production
     */
    public static GProduction parseGProduction(String prodStr, Model model) {
        int index = prodStr.indexOf("->");
        String LHS_str = prodStr.substring(0, index).trim();
        String RHS_str = prodStr.substring(index + 2).trim();
        int bIndex = RHS_str.indexOf("({") + 2;
        int eIndex = RHS_str.lastIndexOf("})");
        RHS_str = RHS_str.substring(bIndex, eIndex).trim();

        StringTokenizer st = new StringTokenizer(RHS_str);

        NonterminalNode LHS_node = model.createNonterminalNode(LHS_str);

        ArrayList<Node> RHS_nodes = new ArrayList<Node>();

        while (st.hasMoreTokens()) {
            String token = st.nextToken();
            Node node;
            if (token.startsWith("*") && !token.startsWith("*^")) {
                node = model.createNonterminalNode(token);
            } else if (LHS_node.getName().equals("*n:Unum") &&
                       Utility.isUnum(token)) {
                node = model.createIntTerminalNode(Integer.parseInt(token));
            } else if (LHS_node.getName().equals("*n:Num") &&
                       Utility.isNum(token)) {
                node = model.createDoubleTerminalNode(Double.parseDouble(token));
            } else {
                node = model.createTerminalNode(token);
            }
            RHS_nodes.add(node);
        }

        GProduction gp = model.createGProduction(LHS_node, RHS_nodes);
        return gp;
    }

    private static class ModelReaderException extends RuntimeException {
		private static final long serialVersionUID = 4168926337449879455L;

		public ModelReaderException(String msg) {
            super(msg);
        }
    }

}
