/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.eval;


/**
 * @author LU, Wei
 * @version 1.0
 */

import hybridtree.core.*;
import hybridtree.core.parse.*;
import hybridtree.core.viterbi.*;
import hybridtree.data.Example;
import hybridtree.data.Model;
import hybridtree.data.TestExample;
import hybridtree.data.TrainExample;
import hybridtree.util.Utility;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.ArrayList;



import java.io.File;

public class PerceptPrepareMix {

    private static String testMode = "parse";

    private static String corpus = "x";

    private static int iteration = 99;

    private static int FOLD_START = 0;

    private static int FOLD_END = 9;

    private static int RUN = 0;

    private static int K = 50;

    private static String set = "none";

    private static String lang = "";

    private static int numTotal = -1;

    private static int numTrain = -1;

    private static void help() {
        System.err.println("Useage: java MixTest [options]");
        System.err.println("\toptions:");
        System.err.println(
                "\t\t corpus=robocup/geofunql (must be specified)");
        System.err.println("\t\t testMode=1,2,.. (default:parse)");
        System.err.println("\t\t run=0-9 (default:0)");
        System.err.println("\t\t folds=e.g.:3-8 (default:0-9)");
        System.err.println("\t\t iteration=10,20,.. (default:100)");
        System.err.println("\t\t gram=unigram/bigram (default:bigram)");
    }

    private static void dispArgs() {
        System.out.println("run:\t" + RUN);
        System.out.println("folds:" + FOLD_START + "~" + FOLD_END);
        System.out.println("corpus:\t" + corpus);
        System.out.println("testMode:\t" + testMode);
        System.out.println("parameter at iteration " + Utility.norm(iteration) +
                           " used.");
    }

    private static void readArgs(String args[]) {
        if (args.length % 2 != 0) {
            System.err.println("Invalid Argument: " + args[args.length - 1]);
            help();
            System.exit(1);
        }

        for (int i = 0; i < args.length; i += 2) {
            String option = args[i];
            String value = args[i + 1];
            if (option.equals("-testMode")) {
                if (value.equals("align")) {
                    testMode = "align";
                } else if (value.equals("parse")) {
                    testMode = "parse";
                } else {
                    System.err.println("Value " + value +
                                       " for testMode not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-run")) {
                try {
                    RUN = Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for run not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-folds")) {
                try {
                    int index = value.indexOf("-");
                    FOLD_START = Integer.parseInt(value.substring(0, index));
                    FOLD_END = Integer.parseInt(value.substring(index + 1));
                } catch (Exception e) {
                    System.err.println("Value " + value +
                                       " for folds not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-corpus")) {
                if (value.equals("robocup")) {
                    corpus = "robocup-clang";
                } else if (value.equals("geofunql")) {
                    corpus = "geo-funql";
                } else if (value.equals("atis")) {
                    corpus = "atis-funql";
                } else {
                    System.err.println("Value " + value +
                                       " for corpus not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-set")) {
                if (value.equals("both") || value.equals("train") ||
                    value.equals("test")) {
                    set = value;
                } else {
                    System.err.println("Value " + value +
                                       " for set not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-numTotal")) {
                try {
                    numTotal = Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for numTotal not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-numTrain")) {
                try {
                    numTrain = Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for numTrain not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-lang")) {
                if (value.equals("en")) {
                    lang = "en";
                } else if (value.equals("es")) {
                    lang = "es";
                } else if (value.equals("ja")) {
                    lang = "ja";
                } else if (value.equals("tr")) {
                    lang = "tr";
                } else {
                    System.err.println("Value " + value +
                                       " for lang not recognizable!");
                    help();
                    System.exit(1);
                }
            }

            else if (option.equals("-iterations")) {
                try {
                    iteration = Integer.parseInt(value)-1;
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for iteration not recognizable!");
                    help();
                    System.exit(1);
                }
            } else {
                System.err.println("Option " + option + " not recoginizable!");
                help();
                System.exit(1);
            }
        }
    }

    public static void main(String args[]) throws IOException,
            ClassNotFoundException {

        readArgs(args);

        dispArgs();

        for (int FOLD = FOLD_START; FOLD <= FOLD_END; FOLD++) {

            String prefix = lang + "/";
            prefix += Utility.norm(numTotal);
            prefix += '-';
            prefix += Utility.norm(numTrain);
            prefix += '/';

            long bTime = System.currentTimeMillis();

            System.err.println("Test..RUN " + RUN + ", FOLD " + FOLD);

            FileInputStream infile1, infile2;
            String filename1 = "param/" + corpus + "/hybridtree-unigram/" + prefix +
                               RUN + "." + FOLD + ".param." +
                               Utility.norm(iteration);
            String filename2 = "param/" + corpus + "/hybridtree-bigram/" + prefix +
                               RUN + "." + FOLD + ".param." +
                               Utility.norm(iteration);

            System.err.println("Reading the Param from: " + filename1);

            infile1 = new FileInputStream(filename1);

            ObjectInputStream ois1 = new ObjectInputStream(infile1);
            Param parameter_unigram = (Param) ois1.readObject();

            infile1.close();

            System.err.println("Reading the Param from: " + filename2);

            infile2 = new FileInputStream(filename2);

            ObjectInputStream ois2 = new ObjectInputStream(infile2);
            Param parameter_bigram = (Param) ois2.readObject();

            infile2.close();

            Model model = parameter_unigram.getModel();

            ParseTable pTable;
            MaxTable mTable;
            TestExample test_ex;

            if (set.equals("train") || set.equals("both")) {

                ArrayList<Example> examples_train = model.getTrainExamples();

                System.err.println("There are totally " + examples_train.size() +
                                   " examples.");
                
                File tmp = new File("result/" + corpus + "/top/hybridtree-mixgram/" + prefix);
                if(!tmp.exists()){
                	tmp.mkdirs();
                }

                File file_train = new File("result/" + corpus +
                                           "/top/hybridtree-mixgram/" + prefix +
                                           Utility.norm(iteration) + "." + RUN +
                                           "." + FOLD +
                                           ".candidates." + Utility.norm(K) +
                                           ".[train]");
                FileOutputStream outfile_train = new FileOutputStream(
                        file_train);

                ObjectOutputStream oos_train = new ObjectOutputStream(
                        outfile_train);

                int corr = 0;

                for (int i = 0; i < examples_train.size(); i++) {
                    TrainExample train_ex = (TrainExample) examples_train.get(i);
                    if (train_ex.getExampleId() < 0) {
                        continue;
                    }
                    test_ex = (train_ex).toTestExample();

                    System.err.println("Example: " + test_ex.getExampleId());

                    try {

                        pTable = ParseTable.createMGTable(test_ex,
                                parameter_unigram, parameter_bigram);
                        mTable = MaxTable.createMGTable(train_ex,
                                parameter_unigram,
                                parameter_bigram);

                        pTable.setCorpus(corpus);

                        mTable.alignMax(K);

                        if (!mTable.ignored() ||
                            mTable.getTopCandidates().size() > 0) {
                            pTable.parseMax(K);

                            if (!pTable.ignore()) {
                                CandidateSet candidates = pTable.
                                        getTopCandidates();
                                CandidateSet corrCandidates = mTable.
                                        getTopCandidates();
                                if (corrCandidates.size() > 0) {
                                    candidates.setCorrectCandidates(
                                            corrCandidates);
                                }

                                for (int k = 0; k < candidates.size(); k++) {
                                    Candidate cand = candidates.getCandidateAt(
                                            k);
                                    if (cand.isCorrect()) {
                                        System.err.println("Good.");
                                        corr++;
                                        break;
                                    }
                                }

                                oos_train.writeObject(candidates);
                                oos_train.flush();
                                System.err.println("[OK]\tExample " +
                                        test_ex.getExampleId() +
                                        " done!");
                                candidates = null;
                                corrCandidates = null;
                                pTable = null;
                                mTable = null;
                            } else {
                                System.err.println("[Warning]\tExample " +
                                        test_ex.getExampleId() +
                                        " ignored.");
                            }
                        } else {
                            System.err.println("[Warning]\tExample " +
                                               test_ex.getExampleId() +
                                               " ignored for mTable.");
                        }
                    } catch (OutOfMemoryError e) {
                        System.err.println("[Warning]\tExample " +
                                           test_ex.getExampleId() +
                                           " ignored due to out of memory error!");
                        System.err.println(e);
                        /*
                         CandidateSet s = new CandidateSet(0);
                                                 oos_train.writeObject(s);
                                                 oos_train.flush();
                         */
                    }

                    parameter_unigram.toString_new();
                    parameter_bigram.toString_new();
                }

                System.out.println("run=" + RUN + ";fold=" + FOLD + ";corr=" +
                                   corr);

                outfile_train.close();

                file_train.setReadOnly();
            }

            if (set.equals("test") || set.equals("both")) {
                ArrayList<Example> examples_test = model.getTestExamples();

                File tmp = new File("result/" + corpus + "/top/hybridtree-mixgram/" + prefix);
                if(!tmp.exists()){
                	tmp.mkdirs();
                }

                File file_test = new File("result/" + corpus +
                                          "/top/hybridtree-mixgram/" + prefix +
                                          Utility.norm(iteration) + "." +
                                          RUN +
                                          "." + FOLD +
                                          ".candidates." + Utility.norm(K) +
                                          ".[test]");
                FileOutputStream outfile_test = new FileOutputStream(
                        file_test);

                ObjectOutputStream oos_test = new ObjectOutputStream(
                        outfile_test);

                for (int i = 0; i < examples_test.size(); i++) {
                    test_ex = (TestExample) examples_test.get(i);

                    System.err.println("Testing Example " +
                                       test_ex.getExampleId());

                    try {

                        pTable = ParseTable.createMGTable(test_ex,
                                parameter_unigram, parameter_bigram);

                        pTable.setCorpus(corpus);

                        pTable.parseMax(K);

                        if (!pTable.ignore()) {
                            System.err.println("[OK]\tExample " +
                                               test_ex.getExampleId() +
                                               " done!");
                        } else {
                            System.err.println("[Warning]\tExample " +
                                               test_ex.getExampleId() +
                                               " ignored.");
                        }
                        CandidateSet candidates = pTable.getTopCandidates();
                        oos_test.writeObject(candidates);
                        oos_test.flush();
                        candidates = null;
                    } catch (OutOfMemoryError e) {
                        System.err.println(e);
                        CandidateSet s = new CandidateSet(0);
                        oos_test.writeObject(s);
                        oos_test.flush();
                    }

                    parameter_unigram.toString_new();
                    parameter_bigram.toString_new();

                }

                outfile_test.close();

                file_test.setReadOnly();

            }

            if (set.equals("none")) {
                System.err.println(
                        "Please specify the data \"set\": train/test");
            }

            long eTime = System.currentTimeMillis();

            System.err.println("Total PrePerceptron Time:\t" +
                               (eTime - bTime) / 1000.0);
        }
    }

}
