/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.eval;


/**
 * @author LU, Wei
 * @version 1.0
 */

import hybridtree.core.*;
import hybridtree.core.perceptron.*;
import hybridtree.util.Pair;
import hybridtree.util.RankedList;
import hybridtree.util.Utility;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.Vector;



import java.io.File;

public class PerceptTrain {

    private static String testMode = "parse";

    private static String gram = "bigram";

    private static String corpus = "x";

    private static int iteration = 99;

    private static int FOLD_START = 0;

    private static int FOLD_END = 9;

    private static int RUN = 0;

    private static int K = 50;

    private static String lang = "";

    private static int numTotal = -1;

    private static int numTrain = -1;


    private static void help() {
        System.err.println("Useage: java PerceptTrain [options]");
        System.err.println("\toptions:");
        System.err.println(
                "\t\t corpus=robocup/geofunql (must be specified)");
        System.err.println("\t\t testMode=1,2,.. (default:parse)");
        System.err.println("\t\t run=0-9 (default:0)");
        System.err.println("\t\t folds=e.g.:3-8 (default:0-9)");
        System.err.println("\t\t iteration=9,19,.. (default:99)");
        System.err.println("\t\t gram=unigram/bigram (default:bigram)");
    }

    private static void dispArgs() {
        System.out.println("run:\t" + RUN);
        System.out.println("folds:" + FOLD_START + "~" + FOLD_END);
        System.out.println("corpus:\t" + corpus);
        System.out.println("testMode:\t" + testMode);
        System.out.println("gram:\t" + gram);
        System.out.println("parameter at iteration " + Utility.norm(iteration) +
                           " used.");
    }

    private static void readArgs(String args[]) {
        if (args.length % 2 != 0) {
            System.err.println("Invalid Argument: " + args[args.length - 1]);
            help();
            System.exit(1);
        }

        for (int i = 0; i < args.length; i += 2) {
            String option = args[i];
            String value = args[i + 1];
            if (option.equals("-testMode")) {
                if (value.equals("align")) {
                    testMode = "align";
                } else if (value.equals("parse")) {
                    testMode = "parse";
                } else {
                    System.err.println("Value " + value +
                                       " for testMode not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-run")) {
                try {
                    RUN = Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for run not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-folds")) {
                try {
                    int index = value.indexOf("-");
                    FOLD_START = Integer.parseInt(value.substring(0, index));
                    FOLD_END = Integer.parseInt(value.substring(index + 1));
                } catch (Exception e) {
                    System.err.println("Value " + value +
                                       " for folds not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-gram")) {
                if (value.equals("unigram")) {
                    gram = "unigram";
                } else if (value.equals("bigram")) {
                    gram = "bigram";
                } else if (value.equals("mixbigram")) {
                    gram = "mixbigram";
                } else if (value.equals("newmixbigram")) {
                    gram = "newmixbigram";
                } else if (value.equals("mixgram")) {
                    gram = "mixgram";
                } else {
                    System.err.println("Value " + value +
                                       " for gram not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-numTotal")) {
                try {
                    numTotal = Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for numTotal not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-numTrain")) {
                try {
                    numTrain = Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for numTrain not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-lang")) {
                if (value.equals("en")) {
                    lang = "en";
                } else if (value.equals("es")) {
                    lang = "es";
                } else if (value.equals("ja")) {
                    lang = "ja";
                } else if (value.equals("tr")) {
                    lang = "tr";
                } else {
                    System.err.println("Value " + value +
                                       " for lang not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-corpus")) {
                if (value.equals("robocup")) {
                    corpus = "robocup-clang";
                } else if (value.equals("geofunql")) {
                    corpus = "geo-funql";
                }else if (value.equals("atis")) {
                    corpus = "atis-funql";
                } else {
                    System.err.println("Value " + value +
                                       " for corpus not recognizable!");
                    help();
                    System.exit(1);
                }
            } else if (option.equals("-iterations")) {
                try {
                    iteration = Integer.parseInt(value) - 1;
                } catch (NumberFormatException e) {
                    System.err.println("Value " + value +
                                       " for iteration not recognizable!");
                    help();
                    System.exit(1);
                }
            } else {
                System.err.println("Option " + option + " not recoginizable!");
                help();
                System.exit(1);
            }
        }
    }

    public static void main(String args[]) throws IOException,
            ClassNotFoundException {

        readArgs(args);

        dispArgs();

        for (int FOLD = FOLD_START; FOLD <= FOLD_END; FOLD++) {

            System.err.println("Run:" + RUN + "\tFold:" + FOLD);

            long bTime = System.currentTimeMillis();

            String prefix = lang + "/";
            prefix += Utility.norm(numTotal);
            prefix += '-';
            prefix += Utility.norm(numTrain);
            prefix += '/';

            File file_read = new File("result/" + corpus + "/top/hybridtree-" + gram +
                                      "/" + prefix + Utility.norm(iteration) +
                                      "." + RUN + "." + FOLD +
                                      ".candidates." + Utility.norm(K) +
                                      ".[train]");
            FileInputStream infile = new FileInputStream(file_read);

            ObjectInputStream ois = new ObjectInputStream(infile);

            Vector<CandidateSet> candidateSets = new Vector<CandidateSet>();
            candidateSets.add((CandidateSet) ois.readObject());
            try {
                while (true) {
                    CandidateSet cands = (CandidateSet) ois.readObject();
                    candidateSets.add(cands);

                    System.err.println("Read " + candidateSets.size() +
                                       " candidateSets!");
                }
            } catch (Exception e) {
            }
            ois.close();
            infile.close();

            Perceptron p = new Perceptron(candidateSets.size());
            System.err.println(candidateSets.size());

            int IT = 10;
            for (int it = 0; it < IT; it++) {
                System.out.println("Iteration " + it);
                for (int i = 0; i < candidateSets.size(); i++) {
                    CandidateSet candidateSet = candidateSets.get(i);
                    if (candidateSet.getCorrectCandidates() == null ||
                        candidateSet.getCorrectCandidates().size() == 0) {
                        continue;
                    }

                    //extract features...
                    if (it == 0) {
                        System.err.println("Feature");
                        for (int rank = 0; rank < candidateSet.size(); rank++) {
                            System.err.print("\rrank\t" + rank);
                            Candidate candidate = candidateSet.getCandidateAt(
                                    rank);
                            candidate.getFeatures(p);
                        }
                        Candidate corrCandidate = candidateSet.
                                                  getCorrectCandidates().
                                                  getCandidateAt(0);
                        corrCandidate.getFeatures(p);
                        System.err.println();
                    }
                    System.err.print("Learn Candidates " + i + "/" +
                                     candidateSets.size());
                    p.perceptLearn(candidateSet);
                    p.perceptAvg(i, it);
                }

                System.err.println("Average");
            }
            p.perceptSwap();
            p.normalizeWeights();

            /*
                         //computing bias...
                         Candidate badCand = null;
                         Candidate goodCand = null;
                         double badCandVal = Double.NEGATIVE_INFINITY;
                         double goodCandVal = Double.POSITIVE_INFINITY;

                         for (int i = 0; i < candidateSets.size(); i++) {
                CandidateSet candidateSet = candidateSets.get(i);
                for (int k = 0; k < candidateSet.size(); k++) {
                    Candidate candidate = candidateSet.getCandidateAt(k);
                    double val = candidate.computeFeatureCost();
                    if (val > badCandVal && !candidate.isCorrect()) {
                        badCandVal = val;
                        badCand = candidate;
                    }
                }
             CandidateSet corrCandidates = candidateSet.getCorrectCandidates();
                Candidate corrCandidate = corrCandidates.getCandidateAt(0);
                if (corrCandidate.computeFeatureCost() < goodCandVal) {
                    goodCandVal = corrCandidate.computeFeatureCost();
                    goodCand = corrCandidate;
                }
                         }

                         p.computeBiasAndMargin(badCand, goodCand);
                         //end of computing bias...
             */



            System.err.println();
            System.err.println("Finding F...");
            RankedList<Candidate, Double> list = new RankedList<Candidate, Double>();

            double totalCorrCandidates = 0;

            for (int i = 0; i < candidateSets.size(); i++) {
                CandidateSet candidateSet = candidateSets.get(i);

                if (candidateSet.getCorrectCandidates() == null) {
                    continue;
                }

                double maxValue = Double.NEGATIVE_INFINITY;
                Candidate maxCandidate = null;
                for (int k = 0; k < candidateSet.size(); k++) {
                    Candidate candidate = candidateSet.getCandidateAt(k);
                    double value = candidate.computeFeatureCost();
                    if (value > maxValue) {
                        maxValue = value;
                        maxCandidate = candidate;
                    }
                }
                if (maxCandidate != null) {
                    list.add(maxCandidate, maxValue);
                    if (maxCandidate.isCorrect()) {
                        totalCorrCandidates++;
                    }
                }
            }

            double totalCandidates = (double) list.size();

            double bestF = totalCorrCandidates / totalCandidates;
            double bias = Double.POSITIVE_INFINITY;
            double margin = 1.0;

            System.err.println("bestF=" + bestF + "\t" + totalCorrCandidates +
                               "/" + totalCandidates);

            for (int i = 0; i < list.size() - 1; i++) {
                Pair<Candidate, Double> candidate_val = list.get(i);
                Candidate candidate = candidate_val.getFirst();
                System.err.println(candidate_val.getSecond() + "\t" +
                                   candidate.isCorrect());
                if (candidate.isCorrect()) {
                    totalCorrCandidates--;
                }

                if (i == 0) {
                    bias = -candidate_val.getSecond();
                }

                if (!candidate.isCorrect()) {

                    Pair<Candidate, Double> candidate_val_next = list.get(i + 1);
                    Candidate candidate_next = candidate_val_next.getFirst();

                    if (candidate_next.isCorrect()) {

                        double precision = totalCorrCandidates /
                                           (totalCandidates - (i + 1));
                        double recall = totalCorrCandidates / (totalCandidates);
                        double f = 2.0 / (1.0 / precision + 1.0 / recall);

                        if (f > bestF) {
                            bestF = f;
                            double val = candidate_val.getSecond();
                            double val_next = candidate_val_next.getSecond();
                            bias = -(val + val_next) / 2;
                            margin = 1.0; //Math.abs(val - val_next) / 2;

                            System.err.println("P=" + precision);
                            System.err.println("R=" + recall);
                            System.err.println("F=" + f);
                            System.err.println("TotalCorr:" +
                                               totalCorrCandidates);
                            System.err.println("TotalCand:" + totalCandidates);
                            System.err.println(val);
                            System.err.println(val_next);
                        }
                    }
                }
            }

            p.setBias(bias);
            p.setMargin(margin);

            System.err.println("bias=" + bias);
            System.err.println("margin=" + margin);
            System.err.println("bestF=" + bestF);

            p.disallowMoreFeatures();

            //System.out.println(p.toString());

            File file_write = new File("result/" + corpus + "/top/hybridtree-" + gram +
                                       "/" + prefix + Utility.norm(iteration) +
                                       "." + RUN + "." + FOLD +
                                       ".percept." + Utility.norm(K));
            FileOutputStream outfile = new FileOutputStream(file_write);
            ObjectOutputStream oos = new ObjectOutputStream(outfile);
            oos.writeObject(p);
            oos.flush();
            oos.close();
            outfile.close();

            long eTime = System.currentTimeMillis();
            System.err.println("Written to:" + file_write);
            System.err.println("Time:" + (eTime - bTime) / 1000.0 + " secs.");

        }
    }

}
