/**
 *  The Hybrid Tree Framework for Semantic Parsing and Language Generation
 *
 *  Copyright (C) 2009  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package hybridtree.util;

import java.util.Vector;
import java.util.List;
import java.util.Collections;

/**
 * @author LU, Wei
 * @version 1.0
 */
@SuppressWarnings("hiding")
public class RankedList<X, Comparable> {

    private Vector<X> _elems;
    private Vector<Comparable> _vals;

    public RankedList() {
        this._elems = new Vector<X>();
        this._vals = new Vector<Comparable>();
    }

    public boolean add(X elem, Comparable val) {
        @SuppressWarnings({ "unchecked", "rawtypes" })
		int index = Collections.binarySearch((List)this._vals, val);
        if (index >= 0) {
            this._elems.add(index, elem);
            this._vals.add(index, val);
        } else {
            index = -index - 1;
            this._elems.add(index, elem);
            this._vals.add(index, val);
        }
        return true;
    }

    public Pair<X, Comparable> get(int pos) {
        X x = this._elems.get(pos);
        Comparable y = this._vals.get(pos);
        return new Pair<X, Comparable>(x, y);
    }

    public int size() {
        return this._elems.size();
    }

}
